// Copyright (C) 2019 ~ 2020 Uniontech Software Technology Co.,Ltd
// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later
#ifndef DDLOG_H
#define DDLOG_H
#include <DConfig>
#include <DLog>
DCORE_USE_NAMESPACE

namespace DDLog {
inline Q_LOGGING_CATEGORY(app, "org.deepin.system-monitor.daemon")
}

#endif   // DDLOG_H
