/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.1
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // settings.json
  0x0,0x0,0x1,0x78,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x5b,0x7,0x75,0xb,0x0,0xc2,0x4e,0x2c,0x1e,0x50,0x6d,0x75,
  0x43,0xc3,0xf,0x22,0xe8,0x56,0x2e,0xb5,0xfd,0x2d,0xdf,0xe4,0x19,0xe9,0x45,0xb1,
  0x1c,0x93,0x53,0xd0,0xc,0x4d,0x6a,0x66,0x67,0x53,0x21,0x4b,0x65,0x51,0x30,0x76,
  0x4f,0x48,0x4,0x36,0xd7,0x1c,0xda,0x5d,0xed,0x69,0x1d,0x13,0x5f,0x88,0xf9,0x6f,
  0x79,0x7c,0xca,0x76,0x1a,0x23,0xa8,0xea,0x65,0x6c,0x7e,0xad,0x9b,0xe6,0x75,0x5,
  0xd8,0x5e,0x40,0x1b,0x6e,0x25,0xf2,0xd,0xdb,0xfb,0x43,0x4c,0x32,0xbe,0x3f,0x82,
  0xc1,0xed,0x6a,0xdd,0xe4,0x12,0x0,0x2,0xca,0x50,0x1c,0x9a,0x19,0xad,0x41,0xe5,
  0xec,0xf6,0x51,0x46,0xb5,0x9b,0xfb,0xdc,0x89,0xa6,0x66,0xc7,0x45,0xc6,0x97,0xac,
  0xd9,0xf9,0xd7,0xe4,0x7a,0x68,0x79,0xea,0x59,0x2b,0x3c,0x93,0x14,0xca,0x95,0x54,
  0x8a,0x8c,0xad,0x6e,0x12,0x37,0x4f,0xae,0xdb,0x28,0x2a,0x14,0xc6,0xd6,0xeb,0x1d,
  0x65,0xf4,0xde,0x3c,0xf1,0xe2,0x6f,0xe1,0x40,0x41,0xb0,0x75,0x9f,0x95,0x32,0x6a,
  0x7a,0x44,0xe9,0xe2,0x50,0x46,0x71,0x34,0xa3,0x84,0x94,0x2e,0x94,0x4d,0xa0,0xe1,
  0xa1,0x52,0x25,0x23,0x15,0x24,0x29,0x66,0xd,0x50,0x48,0x24,0xc6,0x8a,0x1d,0x13,
  0xc6,0x76,0x2d,0x95,0x64,0x79,0x62,0xe4,0xf3,0x8c,0xec,0x96,0x53,0xc4,0xe1,0xd4,
  0x55,0x6b,0x65,0xaa,0xed,0x3a,0xca,0x84,0x4f,0xd2,0xf6,0x3e,0xa,0xde,0x91,0xb2,
  0x3c,0xbf,0x6b,0xef,0x97,0x33,0xb1,0x17,0x67,0xdd,0xda,0xa4,0x40,0x70,0x5a,0x2d,
  0xda,0x2,0x56,0x6c,0x49,0x51,0x81,0x7c,0xad,0x11,0x83,0xda,0x9f,0x50,0xe2,0x47,
  0xe6,0x3d,0x34,0x1c,0xda,0x90,0x33,0x3c,0x83,0x13,0x2c,0xd9,0x2,0xd5,0x2e,0x20,
  0xb9,0x6e,0xd7,0x36,0xad,0x23,0x74,0x92,0x5,0xf8,0xb6,0x6a,0x0,0x4a,0xa1,0x32,
  0x2,0x68,0x2d,0xbd,0xbb,0xb8,0xb0,0x3a,0xa1,0x64,0xe5,0xa8,0x80,0x5c,0x6e,0xe9,
  0x79,0xf9,0xd2,0x5b,0xc7,0x3b,0x18,0x5d,0x49,0xc1,0x20,0xa8,0x62,0xe2,0xc6,0x52,
  0x34,0x6d,0xe6,0x99,0xc8,0xf2,0x57,0x80,0xc5,0x94,0xc3,0xbc,0x90,0xc0,0xe4,0x70,
  0xca,0x66,0x78,0xd7,0xbf,0x63,0x3e,0xe,0xca,0x18,0x4b,0x68,0xe9,0x43,0xa5,0xab,
  0x3a,0x6d,0x86,0x9b,0x0,0xc0,0x3,
  
};

static const unsigned char qt_resource_name[] = {
  // resources
  0x0,0x9,
  0xa,0x6c,0x78,0x43,
  0x0,0x72,
  0x0,0x65,0x0,0x73,0x0,0x6f,0x0,0x75,0x0,0x72,0x0,0x63,0x0,0x65,0x0,0x73,
    // settings.json
  0x0,0xd,
  0xd,0xf2,0x7b,0x3e,
  0x0,0x73,
  0x0,0x65,0x0,0x74,0x0,0x74,0x0,0x69,0x0,0x6e,0x0,0x67,0x0,0x73,0x0,0x2e,0x0,0x6a,0x0,0x73,0x0,0x6f,0x0,0x6e,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0x9e,0x51,0xab,0x43,0x98,
  // :/resources
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0x9e,0x51,0xab,0x43,0x98,
  // :/resources/settings.json
  0x0,0x0,0x0,0x18,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0x9e,0x51,0xab,0x43,0x98,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_deepin_system_monitor_daemon)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_deepin_system_monitor_daemon)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_deepin_system_monitor_daemon)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_deepin_system_monitor_daemon)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_deepin_system_monitor_daemon)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_deepin_system_monitor_daemon)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
