/****************************************************************************
** Meta object code from reading C++ file 'systemmonitorservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/systemmonitorservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'systemmonitorservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20SystemMonitorServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto SystemMonitorService::qt_create_metaobjectdata<qt_meta_tag_ZN20SystemMonitorServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SystemMonitorService",
        "D-Bus Interface",
        "org.deepin.SystemMonitorDaemon",
        "alarmItemChanged",
        "",
        "item",
        "QDBusVariant",
        "value",
        "getSystemProtectionStatus",
        "setSystemProtectionStatus",
        "status",
        "getCpuUsage",
        "getMemoryUsage",
        "getAlarmMsgInterval",
        "setAlarmMsgInterval",
        "interval",
        "getAlarmUsageOfCpu",
        "setAlarmUsageOfCpu",
        "usage",
        "getAlarmUsageOfMemory",
        "setAlarmUsageOfMemory",
        "showDeepinSystemMoniter",
        "changeAlarmItem",
        "setAlaramLastTimeInterval",
        "lastTime",
        "onMonitorTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'alarmItemChanged'
        QtMocHelpers::SignalData<void(const QString &, const QDBusVariant &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'getSystemProtectionStatus'
        QtMocHelpers::SlotData<bool()>(8, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool),
        // Slot 'setSystemProtectionStatus'
        QtMocHelpers::SlotData<void(bool)>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Slot 'getCpuUsage'
        QtMocHelpers::SlotData<int()>(11, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'getMemoryUsage'
        QtMocHelpers::SlotData<int()>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'getAlarmMsgInterval'
        QtMocHelpers::SlotData<int()>(13, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'setAlarmMsgInterval'
        QtMocHelpers::SlotData<void(int)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'getAlarmUsageOfCpu'
        QtMocHelpers::SlotData<int()>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'setAlarmUsageOfCpu'
        QtMocHelpers::SlotData<void(int)>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'getAlarmUsageOfMemory'
        QtMocHelpers::SlotData<int()>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Int),
        // Slot 'setAlarmUsageOfMemory'
        QtMocHelpers::SlotData<void(int)>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'showDeepinSystemMoniter'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'changeAlarmItem'
        QtMocHelpers::SlotData<void(const QString &, const QDBusVariant &)>(22, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Slot 'setAlaramLastTimeInterval'
        QtMocHelpers::SlotData<void(qint64)>(23, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::LongLong, 24 },
        }}),
        // Slot 'onMonitorTimeout'
        QtMocHelpers::SlotData<void()>(25, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<SystemMonitorService, qt_meta_tag_ZN20SystemMonitorServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SystemMonitorService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SystemMonitorServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SystemMonitorServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20SystemMonitorServiceE_t>.metaTypes,
    nullptr
} };

void SystemMonitorService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SystemMonitorService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->alarmItemChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2]))); break;
        case 1: { bool _r = _t->getSystemProtectionStatus();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->setSystemProtectionStatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: { int _r = _t->getCpuUsage();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->getMemoryUsage();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->getAlarmMsgInterval();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->setAlarmMsgInterval((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: { int _r = _t->getAlarmUsageOfCpu();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->setAlarmUsageOfCpu((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: { int _r = _t->getAlarmUsageOfMemory();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->setAlarmUsageOfMemory((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->showDeepinSystemMoniter(); break;
        case 12: _t->changeAlarmItem((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2]))); break;
        case 13: _t->setAlaramLastTimeInterval((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 14: _t->onMonitorTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusVariant >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SystemMonitorService::*)(const QString & , const QDBusVariant & )>(_a, &SystemMonitorService::alarmItemChanged, 0))
            return;
    }
}

const QMetaObject *SystemMonitorService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SystemMonitorService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20SystemMonitorServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int SystemMonitorService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void SystemMonitorService::alarmItemChanged(const QString & _t1, const QDBusVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
