/****************************************************************************
** Meta object code from reading C++ file 'service_manager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../service/service_manager.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'service_manager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11CustomTimerE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomTimer::qt_create_metaobjectdata<qt_meta_tag_ZN11CustomTimerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomTimer"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomTimer, qt_meta_tag_ZN11CustomTimerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomTimer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CustomTimerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CustomTimerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11CustomTimerE_t>.metaTypes,
    nullptr
} };

void CustomTimer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomTimer *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomTimer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomTimer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11CustomTimerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int CustomTimer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14ServiceManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto ServiceManager::qt_create_metaobjectdata<qt_meta_tag_ZN14ServiceManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ServiceManager",
        "errorOccurred",
        "",
        "ErrorContext",
        "ec",
        "beginUpdateList",
        "serviceListUpdated",
        "QList<SystemServiceEntry>",
        "list",
        "serviceStatusUpdated",
        "SystemServiceEntry",
        "entry",
        "startService",
        "id",
        "param",
        "stopService",
        "restartService",
        "setServiceStartupMode",
        "autoStart"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'errorOccurred'
        QtMocHelpers::SignalData<void(const ErrorContext &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'beginUpdateList'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'serviceListUpdated'
        QtMocHelpers::SignalData<void(const QList<SystemServiceEntry> &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'serviceStatusUpdated'
        QtMocHelpers::SignalData<void(const SystemServiceEntry &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'startService'
        QtMocHelpers::SlotData<ErrorContext(const QString &, const QString &)>(12, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'startService'
        QtMocHelpers::SlotData<ErrorContext(const QString &)>(12, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'stopService'
        QtMocHelpers::SlotData<ErrorContext(const QString &)>(15, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'restartService'
        QtMocHelpers::SlotData<ErrorContext(const QString &, const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 },
        }}),
        // Slot 'restartService'
        QtMocHelpers::SlotData<ErrorContext(const QString &)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 3, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'setServiceStartupMode'
        QtMocHelpers::SlotData<ErrorContext(const QString &, bool)>(17, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 13 }, { QMetaType::Bool, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ServiceManager, qt_meta_tag_ZN14ServiceManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ServiceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ServiceManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ServiceManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ServiceManagerE_t>.metaTypes,
    nullptr
} };

void ServiceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ServiceManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->errorOccurred((*reinterpret_cast<std::add_pointer_t<ErrorContext>>(_a[1]))); break;
        case 1: _t->beginUpdateList(); break;
        case 2: _t->serviceListUpdated((*reinterpret_cast<std::add_pointer_t<QList<SystemServiceEntry>>>(_a[1]))); break;
        case 3: _t->serviceStatusUpdated((*reinterpret_cast<std::add_pointer_t<SystemServiceEntry>>(_a[1]))); break;
        case 4: { ErrorContext _r = _t->startService((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        case 5: { ErrorContext _r = _t->startService((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        case 6: { ErrorContext _r = _t->stopService((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        case 7: { ErrorContext _r = _t->restartService((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        case 8: { ErrorContext _r = _t->restartService((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        case 9: { ErrorContext _r = _t->setServiceStartupMode((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<ErrorContext*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ErrorContext >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ServiceManager::*)(const ErrorContext & )>(_a, &ServiceManager::errorOccurred, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServiceManager::*)()>(_a, &ServiceManager::beginUpdateList, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServiceManager::*)(const QList<SystemServiceEntry> & )>(_a, &ServiceManager::serviceListUpdated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServiceManager::*)(const SystemServiceEntry & )>(_a, &ServiceManager::serviceStatusUpdated, 3))
            return;
    }
}

const QMetaObject *ServiceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ServiceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ServiceManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ServiceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void ServiceManager::errorOccurred(const ErrorContext & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ServiceManager::beginUpdateList()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ServiceManager::serviceListUpdated(const QList<SystemServiceEntry> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ServiceManager::serviceStatusUpdated(const SystemServiceEntry & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
